extends Node
## The singleton of all singletons

@onready var pause_menu := $PauseMenu/Root
@onready var continue_button := $PauseMenu/Root/Panel/VBoxContainer/continue

func _ready():
	pause_menu.hide()

func _process(_delta):
	if Input.is_action_just_pressed("toggle_fullscreen"):
		var new_mode
		if DisplayServer.window_get_mode() == DisplayServer.WINDOW_MODE_FULLSCREEN:
			new_mode = DisplayServer.WINDOW_MODE_WINDOWED
		else:
			new_mode = DisplayServer.WINDOW_MODE_FULLSCREEN
		DisplayServer.window_set_mode(new_mode)
	if Input.is_action_just_pressed("ui_cancel") && pause_menu.visible:
		toggle_pause()
	if Input.is_action_just_pressed("pause"):
		toggle_pause()
		if pause_menu.visible:
			continue_button.grab_focus()

var current_player: Node = null
var player_respawn_position: Vector2
## Call this from Player, makes sure that the player is a child of root and
## reccords the position for debug-respawning.
func player_init(player: Node):
	player_respawn_position = player.position
	if player != current_player:
		current_player = player
		player.get_parent().remove_child.call_deferred(player)
		get_node("/root").add_child.call_deferred(player)
	else:
		player.queue_free()


# Saving: settings and in-game-saves are handled differently
var settings := {
	"vsync" : true,
}

## Saves settings
func apply_settings():
	var json_string = JSON.stringify(settings)
	var settings_file = FileAccess.open("user://settings.json", FileAccess.WRITE)
	settings_file.store_string(json_string)
	settings_file.close()

func load_settings():
	var settings_file = FileAccess.open("user://settings.json", FileAccess.READ)
	var json_string = settings_file.get_line()
	var json = JSON.new()
	var parse_result = json.parse(json_string)
	if not parse_result == OK:
		printerr("Failed to load settings")
		return
	settings = parse_result.data
	settings_file.close()

func toggle_pause():
	pause_menu.visible = !pause_menu.visible
	get_tree().paused = pause_menu.visible

func _on_exit_game_pressed():
	get_tree().quit()

func _on_vsync_toggled(toggled_on: bool) -> void:
	settings["vsync"] = toggled_on
	DisplayServer.window_set_vsync_mode(DisplayServer.VSYNC_ENABLED if toggled_on else DisplayServer.VSYNC_DISABLED)
	apply_settings()
